/**
 * Created by Grigorii Shovkoplias on 06.02.2016.
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        new File("../preliminary").mkdir();
        desc = new PrintWriter("../tests.desc");


        done = 0;
        genSamples();

        //-------------------------first group-------------------------------------------------
        subtask = "subtask1";

        for (int i = 0; i < 5; i++) {
            genRandGroup1(MAXN1);
        }

        genManual(1, MAXN1);
        genManual(7, MAXN1 - 4);

        desc.println(String.format("comment_subtask%d=\"n &lt;= 9 and m &lt;= %d\"", 1, MAXN1));
        desc.println(String.format("bonus_subtask%d=%d", 1, 27));

        //-------------------------second group-------------------------------------------------
        subtask = "subtask2";

        for (int i = 0; i < 3; i++) {
            genRand(MAXN2);
            genRandGroup1(MAXN2);
            genRandSpecial(MAXN2, rand(MAXN1));
        }

        genManual(1, MAXN2);
        genManual(7, MAXN2 - 4);

        desc.println(String.format("comment_subtask%d=\"m &lt;= %d\"", 2, MAXN2));
        desc.println(String.format("bonus_subtask%d=%d", 2, 27));

        //-------------------------third group-------------------------------------------------
        subtask = "subtask3";

        for (int i = 0; i < 5; i++) {
            genRand(MAXN3);
            genRandGroup1(MAXN3);
            genRandSpecial(MAXN3, rand(MAXN1));
        }

        for (int i = 10; i <= MAXN3; i *= 10) {
            genSpecial(MAXN3, i);
        }

        genManual(1, MAXN3);
        genManual(7, MAXN3 - 4);
        genManual(13, MAXN3 - 100);
        genManual(24, MAXN3);

        desc.println(String.format("comment_subtask%d=\"m &lt;= %d\"", 3, MAXN2));
        desc.println(String.format("bonus_subtask%d=%d", 3, 46));


        desc.close();

    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private PrintWriter desc;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN1 = 1000;
    private final int MAXN2 = 100_000;
    private final int MAXN3 = 200_000_000;



    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private int gand(int n) {
        return rand(n / 2) + n / 2 + 1;
    }

    private char zeroOrOne() {
        return (char) ('0' + rnd.nextInt(2));
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }

    private void print(int n, int m) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(n + " " + m);
        out.close();
    }


    private String getPath() {
        return String.format("../tests/%02d", done);
    }

    private void printDesc(String path, String folder, String subtask, int points, String comment) throws IOException {
        desc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, points, comment));
    }

    private String subtask;

    private void printDescT(String comment) throws IOException {
        printDesc(getPath(), "tests", subtask, 0, comment);
    }

    //begin generators

    private void genSamples() throws IOException {
        PrintWriter out;
        String path;
        path = "../preliminary/01";
        out = new PrintWriter(path);
        out.println("5 15");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 1");
        path = "../preliminary/02";
        out = new PrintWriter(path);
        out.println("1 100");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 2");
        path = "../preliminary/03";
        out = new PrintWriter(path);
        out.println("23 122");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 3");
    }

    //begin generators

    private void genRand(int max) throws IOException {
        int m = rand(max) + 1;
        int n = rand(m) + 1;
        print(n, m);
        printDescT(String.format("Random: n = %d, m = %d", n, m));
    }

    private void genManual(int n, int m) throws IOException {
        print(n, m);
        printDescT(String.format("Manual: n = %d, m = %d", n, m));
    }

    private void genRandGroup1(int max) throws IOException {
        int m = rand(max) + 1;
        int n = rand(9) + 1;
        print(n, m);
        printDescT(String.format("Random group 1: n = %d, m = %d", n, m));
    }

    private void genRandSpecial(int max, int devider) throws IOException {
        int m = rand(max) + 1;
        int n = rand(m / devider) + 1;
        print(n, m);
        printDescT(String.format("Random Special: n = %d, m = %d", n, m));
    }

    private void genSpecial(int m, int devider) throws IOException {
        int n = rand(m / devider) + 1;
        print(n, m);
        printDescT(String.format("Special: n = %d, m = %d", n, m));
    }

    //end generators

}
